module eposcorr_sel

  use param  
  use dal
  use eposcorr_def

  implicit none

  private findmindist

  public preselectlist

contains

 !--------------------------------------------------------------------
  subroutine preselectlist( xraylist, nxray, optlist, nopt, maxdist )
 !--------------------------------------------------------------------
    type( Coordinate_type), dimension(:), intent(in out) :: xraylist, optlist
    integer(kind=int32), intent(in out) :: nxray, nopt
    real(kind=single),  intent(in) :: maxdist

    integer(kind=int32) :: i, nnewopt, crossid
    real(kind=single) :: error, ra_err, dec_err, mindist
    real(kind=double) :: ra, dec, lii_corr, bii_corr


    if( nopt <= 2 .OR. nxray <= 2 ) then
       call message(AppMsg,Verbose,&
            'Source list is very short. No preselection performed')
       return
    end if


    call findmindist( xraylist, nxray, optlist, nopt )

    nnewopt = nopt

    do i=1,nopt
       if( optlist(i)%mindist > maxdist .AND. i < nnewopt) then
          ! interchange row i and the last row (i.e. with rownr nnewopt)
          ! but first check if for the last row(s) mindist < maxdist
          do while( nnewopt > i+1 .AND.  optlist(nnewopt)%mindist > maxdist)
             nnewopt = nnewopt -1
          end do

          crossid = optlist(nnewopt)%crossid
          error   = optlist(nnewopt)%error
          ra_err  = optlist(nnewopt)%ra_err
          dec_err = optlist(nnewopt)%dec_err
          mindist = optlist(nnewopt)%mindist
          ra      = optlist(nnewopt)%ra
          dec     = optlist(nnewopt)%dec
          lii_corr = optlist(nnewopt)%lii_corr
          bii_corr = optlist(nnewopt)%bii_corr

          optlist(nnewopt)%crossid = optlist(i)%crossid
          optlist(nnewopt)%error   = optlist(i)%error
          optlist(nnewopt)%ra_err  = optlist(i)%ra_err
          optlist(nnewopt)%dec_err = optlist(i)%dec_err
          optlist(nnewopt)%mindist = optlist(i)%mindist
          optlist(nnewopt)%ra      = optlist(i)%ra
          optlist(nnewopt)%dec     = optlist(i)%dec
          optlist(nnewopt)%lii_corr = optlist(i)%lii_corr
          optlist(nnewopt)%bii_corr = optlist(i)%bii_corr

          optlist(i)%crossid = crossid
          optlist(i)%error   = error
          optlist(i)%ra_err  = ra_err
          optlist(i)%dec_err = dec_err
          optlist(i)%mindist = mindist
          optlist(i)%ra      = ra
          optlist(i)%dec     = dec
          optlist(i)%lii_corr = lii_corr
          optlist(i)%bii_corr = bii_corr

          nnewopt = nnewopt -1
       end if

    end do

    nopt= nnewopt

  end subroutine preselectlist


 !--------------------------------------------------------------------
  subroutine findmindist( xraylist, nxray, optlist, nopt )
 !--------------------------------------------------------------------
 ! Finds the minimum distance between an optical source and an x-ray
 ! source. The distance and the corresponding x-ray source id are stored.
 !
    type( Coordinate_type), dimension(:), intent(in out) :: xraylist, optlist
    integer(kind=int32), intent(in) :: nxray, nopt

    integer(kind=int32) :: i,j
    real(kind=double) :: deltaRA, angdist, cosdist

    
    do i=1,nopt

       optlist(i)%mindist = 360.0*3600.0
       optlist(i)%crossid = -1
       do j=1,nxray

          deltaRA = (xraylist(j)%ra - optlist(i)%ra)
          cosdist = dsin(xraylist(j)%dec) * dsin(optlist(i)%dec) + &
               dcos(xraylist(j)%dec) * dcos(optlist(i)%dec) * dcos(deltaRA)
          ! This is strange but necessary!!!
          if( cosdist > 0.9 ) cosdist = min(1.0d0, cosdist) 
          if( cosdist < -0.9 ) cosdist = max(-1.0d0, cosdist) 
          
          angdist = dacos(cosdist)*rad2arcsec
          if( optlist(i)%crossid == -1 .OR. angdist < optlist(i)%mindist) then
              optlist(i)%mindist = angdist
              optlist(i)%crossid = j
          end if
          
       end do

    end do


  end subroutine findmindist


end module eposcorr_sel
